package widgets

import (
	"fmt"

	//	"../util"
	"github.com/awesome-gocui/gocui"
)

type CheckboxWidget struct {
	Widget
	text    string
	checked bool
}

func NewCheckboxWidget(g *gocui.Gui, parent *Window, name string, x, y, width int, text string, checked bool) *CheckboxWidget {
	return &CheckboxWidget{
		Widget: Widget{
			gui:       g,
			parent:    parent,
			name:      name,
			rotatable: true,
			x:         x, y: y,
			width: width, height: 2,
			elementType: "checkbox",
		},
		text:    text,
		checked: checked,
	}
}

func (w *CheckboxWidget) SetCheck(check bool) {
	w.checked = check
	if w.view != nil {
		w.view.SetCursor(0, 0)
		w.view.SetOrigin(0, 0)
		w.view.Rewind()
		w.view.Clear()
		if w.checked {
			fmt.Fprint(w.view, "x")
		} else {
			fmt.Fprint(w.view, " ")
		}
	}
}

func (w *CheckboxWidget) IsChecked() bool {
	return w.checked
}

func (w *CheckboxWidget) Deactivate() {
	w.gui.Cursor = false
	if w.view != nil {
		w.view.Highlight = false
	}
}

func (w *CheckboxWidget) Activate() {
	w.gui.Cursor = true
	if w.view != nil {
		w.view.Highlight = true
		w.gui.SetCurrentView(w.name)
	}
}

func (w *CheckboxWidget) Checker(g *gocui.Gui, v *gocui.View) error {
	w.checked = !w.checked
	if w.view != nil {
		w.view.SetCursor(0, 0)
		w.view.SetOrigin(0, 0)
		w.view.Rewind()
		w.view.Clear()
		if w.checked {
			fmt.Fprint(w.view, "x")
		} else {
			fmt.Fprint(w.view, " ")
		}
		w.OnChange()
	}
	return nil
}

func (w *CheckboxWidget) Layout(g *gocui.Gui) error {
	var err error
	if w.parent != nil {
		w.y = w.parent.y + w.parent.currentHeight
		w.x = w.parent.x + 1
		w.width = w.parent.GetWidth()
	}
	v1, err := g.SetView(w.name+"lbl1", w.x, w.y, w.x+2, w.y+w.height, 0)
	if err != nil {
		if err != gocui.ErrUnknownView {
			return err
		}
		v1.Frame = false
		v1.Editable = false
		v1.FgColor = gocui.ColorWhite | gocui.AttrBold
		v1.SelFgColor = gocui.ColorWhite | gocui.AttrBold
		//fmt.Fprintf(v, fmt.Sprintf("[ ] %s", w.text))
		fmt.Fprint(v1, "[")
	}
	// 45 -                 - 85
	v2, err := g.SetView(w.name+"lbl2", w.x+2, w.y, w.x+w.width, w.y+w.height, 0)
	if err != nil {
		if err != gocui.ErrUnknownView {
			return err
		}
		v2.Frame = false
		v2.Editable = false
		v2.FgColor = gocui.ColorWhite | gocui.AttrBold
		v2.SelFgColor = gocui.ColorWhite | gocui.AttrBold
		//fmt.Fprintf(v, fmt.Sprintf("[ ] %s", w.text))
		fmt.Fprint(v2, fmt.Sprintf("] %s", w.text))
	}

	w.view, err = g.SetView(w.name, w.x+1, w.y, w.x+3, w.y+w.height, 0)
	if err != nil {
		if err != gocui.ErrUnknownView {
			return err
		}
		w.view.Frame = false
		w.view.Editable = false
		w.view.BgColor = gocui.ColorBlack
		w.view.FgColor = gocui.ColorWhite
		w.view.SelFgColor = gocui.ColorBlack
		w.view.SelBgColor = gocui.ColorWhite
		if w.checked {
			fmt.Fprint(w.view, "x")
		} else {
			fmt.Fprint(w.view, " ")
		}
		if w.parent != nil && !w.tabBound {
			w.tabBound = true
			w.gui.SetKeybinding(w.name, gocui.KeyTab, gocui.ModNone, w.parent.Rotator)
			w.gui.SetKeybinding(w.name, gocui.KeyArrowDown, gocui.ModNone, w.parent.Rotator)
			w.gui.SetKeybinding(w.name, gocui.KeyArrowUp, gocui.ModNone, w.parent.RotatorBack)
		}
		w.gui.SetKeybinding(w.name, "x", gocui.ModNone, w.Checker)
		w.gui.SetKeybinding(w.name, gocui.KeyEnter, gocui.ModNone, w.Checker)
		w.gui.SetKeybinding(w.name, gocui.KeySpace, gocui.ModNone, w.Checker)

	}

	return nil
}
