package widgets

import (
	"fmt"
	"github.com/awesome-gocui/gocui"
	"strings"
)

type LabelWidget struct {
	Widget
	text string
}

func NewLabelWidget(g *gocui.Gui, parent *Window, name string, x, y, width int, text string) *LabelWidget {
	return &LabelWidget{
		Widget: Widget{
			gui:    g,
			parent: parent,
			name:   name,
			x:      x, y: y,
			width: width, height: 2,
			elementType: "label",
		},
		text: text,
	}
}

func (w *LabelWidget) SetText(text string) {
	fmt.Fprint(w.view, text)
}

func (w *LabelWidget) Layout(g *gocui.Gui) error {
	var err error
	if w.parent != nil {
		w.y = w.parent.y + w.parent.currentHeight
		w.x = w.parent.x
		w.width = w.parent.GetWidth() + 1
	}

	w.view, err = g.SetView(w.name, w.x+1, w.y, w.x+w.width-1, w.y+w.height, 0)
	if err != nil {
		if err != gocui.ErrUnknownView {
			return err
		}
		if w.alignment == AlignCenter {
			canvasSize := w.width - len(w.text)
			paddingLength := canvasSize / 2
			fmt.Fprint(w.view, w.leftPad(w.text, " ", paddingLength))
		} else {
			fmt.Fprint(w.view, w.text)
		}

		//w.view.BgColor = gocui.ColorWhite |gocui.AttrBold
		w.view.FgColor = gocui.ColorWhite | gocui.AttrBold
		w.view.BgColor = gocui.ColorBlack
		w.view.Editable = false
		w.view.Frame = false
	}
	return nil
}

func (w *LabelWidget) leftPad(s string, padStr string, pLen int) string {
	if pLen > 0 {
		return strings.Repeat(padStr, pLen) + s
	} else {
		return s
	}

}
