package widgets

import (
	"github.com/awesome-gocui/gocui"
)

const (
	AlignLeft int = iota
	AlignCenter
	AlignRight
)

type Widget struct {
	gui           *gocui.Gui
	view          *gocui.View
	parent        *Window
	rotatable     bool
	tabBound      bool
	name          string
	x             int
	y             int
	width         int
	height        int
	needCursor    bool
	alignment     int
	buffer        string
	elementType   string
	padding       int
	isVisibleFunc func() bool
	onChange      func()
}

func (w *Widget) GetElementType() string {
	return w.elementType
}

func (w *Widget) SetPadding(padding int) {
	w.padding = padding
}

func (w *Widget) GetPadding() int {
	return w.padding
}

func (w *Widget) SetOnChangeFunc(ocf func()) {
	w.onChange = ocf
}

func (w *Widget) OnChange() {
	if w.onChange != nil {
		w.onChange()
	}
}

func (w *Widget) SetIsVisibleFunc(ivf func() bool) {
	w.isVisibleFunc = ivf
}

func (w *Widget) Visible() bool {
	if w.isVisibleFunc != nil {
		return w.isVisibleFunc()
	}
	return true
}

func (w *Widget) SetAlignment(alignment int) {
	w.alignment = alignment
}

func (w *Widget) SetBuffer(s string) {
	w.buffer = s
}

func (w *Widget) GetBuffer() string {
	if w.view == nil {
		return ""
	}
	return w.view.ViewBuffer()
}

func (w *Widget) SetParent(p *Window) {
	w.parent = p
}

func (w *Widget) IsRotatable() bool {
	return w.rotatable
}

func (w *Widget) GetName() string {
	if w.view != nil {
		return w.view.Name()
	}
	return ""
}

func (w *Widget) GetWidth() int {
	x, _ := w.view.Size()
	return x
}

func (w *Widget) GetHeight() int {
	return w.height
}

func (w *Widget) AddHandler(key interface{}, handler func(g *gocui.Gui, v *gocui.View) error) {
	w.gui.DeleteKeybinding(w.name, key, gocui.ModNone)
	w.gui.SetKeybinding(w.name, key, gocui.ModNone, handler)
}

func (w *Widget) Activate() {
	//w.gui.Cursor = w.needCursor
	w.gui.SetCurrentView(w.name)
}

func (w *Widget) Deactivate() {
	w.gui.DeleteKeybindings(w.view.Name())
	w.gui.DeleteView(w.view.Name())
}

// Flush - Replaces the hack of exposing the internal gocui flush() method and calling it directly.
// Flush will queue a noop user event which will cause a Layout(). The user event will be
// queued (i.e. this is not immediate as the hack was), but should still work as intended.
func (w *Widget) Flush() {
	if w.gui != nil {
		w.gui.Update(func(*gocui.Gui) error {
			return nil
		})
	}
}
